@@ -34,6 +34,7 @@ int loadcfg(const char *cfgfile, const ConfigType type)
 		 {"BandwidthDetection", 0, &cfg.bwdetection, 0, 0},
 		 {"MaxBandwidth", 0, &cfg.maxbw, 0, 0},
 		 {"Sampletime", 0, &cfg.sampletime, 0, 0},
+		 {"LiveSpinner", 0, &cfg.livespinner, 0, 0},
 		 {"QueryMode", 0, &cfg.qmode, 0, 0},
 		 {"CheckDiskSpace", 0, &cfg.spacecheck, 0, 0},
 		 {"BootVariation", 0, &cfg.bvar, 0, 0},
@@ -97,10 +98,12 @@ int loadcfg(const char *cfgfile, const ConfigType type)
 		 {"CText", cfg.ctext, 0, 8, 0},
 		 {"CLine", cfg.cline, 0, 8, 0},
 		 {"CLineL", cfg.clinel, 0, 8, 0},
+		 {"CPercentileLine", cfg.cpercentileline, 0, 8, 0},
 		 {"CRx", cfg.crx, 0, 8, 0},
 		 {"CRxD", cfg.crxd, 0, 8, 0},
 		 {"CTx", cfg.ctx, 0, 8, 0},
 		 {"CTxD", cfg.ctxd, 0, 8, 0},
+		 {"CTotal", cfg.ctotal, 0, 8, 0},
 		 {"Experimental", 0, &cfg.experimental, 0, 0},
 		 {0, 0, 0, 0, 0}};
 
@@ -199,6 +202,7 @@ void validatecfg(const ConfigType type)
 	validateint("HourlySectionStyle", &cfg.hourlystyle, HOURLYSTYLE, 0, 3);
 	validateint("BootVariation", &cfg.bvar, BVAR, 0, 300);
 	validateint("Sampletime", &cfg.sampletime, DEFSAMPTIME, 2, 600);
+	validatebool("LiveSpinner", &cfg.livespinner, LIVESPINNER);
 	validateint("MonthRotate", &cfg.monthrotate, MONTHROTATE, 1, 28);
 	validatebool("MonthRotateAffectsYears", &cfg.monthrotateyears, MONTHROTATEYEARS);
 	validateint("MaxBandwidth", &cfg.maxbw, DEFMAXBW, 0, BWMAX);
@@ -340,6 +344,7 @@ void defaultcfg(void)
 	cfg.bvar = BVAR;
 	cfg.qmode = DEFQMODE;
 	cfg.sampletime = DEFSAMPTIME;
+	cfg.livespinner = LIVESPINNER;
 	cfg.monthrotate = MONTHROTATE;
 	cfg.monthrotateyears = MONTHROTATEYEARS;
 	cfg.unitmode = UNITMODE;
@@ -431,10 +436,12 @@ void defaultcfg(void)
 	strncpy_nt(cfg.ctext, CTEXT, 8);
 	strncpy_nt(cfg.cline, CLINE, 8);
 	strncpy_nt(cfg.clinel, CLINEL, 8);
+	strncpy_nt(cfg.cpercentileline, CPERCENTILELINE, 8);
 	strncpy_nt(cfg.crx, CRX, 8);
 	strncpy_nt(cfg.crxd, CRXD, 8);
 	strncpy_nt(cfg.ctx, CTX, 8);
 	strncpy_nt(cfg.ctxd, CTXD, 8);
+	strncpy_nt(cfg.ctotal, CTOTAL, 8);
 
 	cfg.timestampprints = 0;
 	cfg.experimental = 0;
