@@ -85,9 +85,13 @@ void printcfgfile(void)
 	defaultcomment(cfg.sampletime == DEFSAMPTIME);
 	printf("Sampletime %d\n\n", cfg.sampletime);
 
+	printf("# show animation at the beginning of -l / --live line (1 = enabled, 0 = disabled)\n");
+	defaultcomment(cfg.livespinner == LIVESPINNER);
+	printf("LiveSpinner %d\n\n", cfg.livespinner);
+
 	printf("# default query mode\n");
-	printf("# 0 = normal, 1 = days, 2 = months, 3 = top, 5 = short\n");
-	printf("# 7 = hours, 8 = xml, 9 = one line, 10 = json\n");
+	printf("# 0 = summary, 1 = days, 2 = months, 3 = top, 4 = single summary, 5 = short\n");
+	printf("# 7 = hours graph, 8 = xml, 9 = one line, 10 = json, 11 = hours, 12 = 5 minute\n");
 	defaultcomment(cfg.qmode == DEFQMODE);
 	printf("QueryMode %d\n\n", cfg.qmode);
 
@@ -340,6 +344,8 @@ void printcfgfile(void)
 	printf("CLine           \"%s\"\n", cfg.cline);
 	defaultcomment(strcmp(cfg.clinel, CLINEL) == 0);
 	printf("CLineL          \"%s\"\n", cfg.clinel);
+	defaultcomment(strcmp(cfg.cpercentileline, CPERCENTILELINE) == 0);
+	printf("CPercentileLine \"%s\"\n", cfg.cpercentileline);
 	defaultcomment(strcmp(cfg.crx, CRX) == 0);
 	printf("CRx             \"%s\"\n", cfg.crx);
 	defaultcomment(strcmp(cfg.ctx, CTX) == 0);
@@ -348,6 +354,8 @@ void printcfgfile(void)
 	printf("CRxD            \"%s\"\n", cfg.crxd);
 	defaultcomment(strcmp(cfg.ctxd, CTXD) == 0);
 	printf("CTxD            \"%s\"\n", cfg.ctxd);
+	defaultcomment(strcmp(cfg.ctotal, CTOTAL) == 0);
+	printf("CTotal          \"%s\"\n", cfg.ctotal);
 }
 
 void defaultcomment(const int isdefault)
