@@ -89,6 +89,10 @@ and most can be changed later from the config file.
 #define DATETIMEFORMAT "%Y-%m-%d %H:%M:%S"
 #endif
 
+#ifndef DATETIMEFORMATWITHOUTSECS
+#define DATETIMEFORMATWITHOUTSECS "%Y-%m-%d %H:%M"
+#endif
+
 /* characters used for visuals */
 #define RXCHAR "%"
 #define TXCHAR ":"
@@ -147,6 +151,9 @@ and most can be changed later from the config file.
 /* 0 = feature disabled */
 #define DEFMAXBW 1000
 
+/* animation visibility in -l / --live */
+#define LIVESPINNER 1
+
 /* maximum allowed config value for bandwidth */
 #define BWMAX 50000
 
@@ -205,7 +212,7 @@ and most can be changed later from the config file.
 
 /* json format version */
 /* 1 = 1.13-1.16, 2 = 2.0 */
-#define JSONVERSION 2
+#define JSONVERSION "2"
 
 /* json format version, -tr */
 /* 1 = 1.18- */
@@ -215,6 +222,10 @@ and most can be changed later from the config file.
 /* 1 = 1.18- */
 #define JSONVERSION_LIVE 1
 
+/*json format version, --alert */
+/* 1 = 2.12- */
+#define JSONALERT "1"
+
 /* --oneline format version */
 #define ONELINEVERSION 1
 
@@ -302,10 +313,15 @@ and most can be changed later from the config file.
 #define CTEXT "000000"
 #define CLINE "B0B0B0"
 #define CLINEL "-"
+#define CPERCENTILELINE "CF0045"
 #define CRX "92CF00"
 #define CRXD "-"
 #define CTX "606060"
 #define CTXD "-"
+#define CTOTAL "0098CF"
+
+/* percentile month entry count */
+#define PERCENTILEENTRYCOUNT 744
 
 /* interface name length limit */
 #define MAXIFLEN 32
@@ -316,6 +332,9 @@ and most can be changed later from the config file.
 /* a full disk (as reported by sqlite) will no cause retries or exit */
 #define DBRETRYLIMIT 5
 
+/* buffer sizes */
+#define DATEBUFFLEN 64
+
 /* internal config structure */
 typedef struct {
 	char dformat[64], mformat[64], tformat[64], hformat[64];
@@ -324,12 +343,12 @@ typedef struct {
 	char dbdir[512], dbtzmodifier[14];
 	char rxchar[2], txchar[2], rxhourchar[2], txhourchar[2], estimatetext[10];
 	char cbg[8], cedge[8], cheader[8], cheadertitle[8], cheaderdate[8], ctext[8];
-	char cline[8], clinel[8], cvnstat[8], crx[8], crxd[8], ctx[8], ctxd[8];
+	char cline[8], clinel[8], cpercentileline[8], cvnstat[8], crx[8], crxd[8], ctx[8], ctxd[8], ctotal[8];
 	int32_t unitmode, rateunitmode, rateunit, bvar, qmode, ifacematchmethod, sampletime, hourlyrate, summaryrate;
 	int32_t monthrotate, monthrotateyears, maxbw, spacecheck, trafficlessentries, transbg, ostyle;
 	int32_t defaultdecimals, hourlydecimals, hourlystyle, is64bit, waldb, dbsynchronous, useutc, imagescale;
 	int32_t largefonts, linespaceadjust, estimatebarvisible, estimatestyle, estimatevisible, barshowsrate, fivegresultcount;
-	int32_t fivegheight, summarygraph, hourlygmode, alwaysadd;
+	int32_t fivegheight, summarygraph, hourlygmode, alwaysadd, livespinner;
 	char cfgfile[512], logfile[512], pidfile[512];
 	char daemonuser[33], daemongroup[33];
 	int32_t timesyncwait, updateinterval, pollinterval, saveinterval, offsaveinterval, rescanonsave, savestatus;
