@@ -269,7 +269,7 @@ void initdstate(DSTATE *s)
 	debug = 0;		   /* debug disabled by default */
 	disableprinte = 0; /* let printe() output be visible */
 	stderrprinte = 0;  /* use stdout for printe() output */
-	s->rundaemon = 0;  /* daemon disabled by default */
+	s->rundaemon = -1;  /* daemon disabled by default */
 
 	s->running = 0;
 	s->dbsaved = 1;
@@ -277,7 +277,9 @@ void initdstate(DSTATE *s)
 	s->sync = 0;
 	s->forcesave = 0;
 	s->noadd = 0;
+	s->noremove = 0;
 	s->initdb = 0;
+	s->startempty = 0;
 	s->iflisthash = 0;
 	s->cfgfile[0] = '\0';
 	s->user[0] = '\0';
@@ -300,18 +302,12 @@ void preparedatabase(DSTATE *s)
 {
 	s->dbifcount = db_getinterfacecount();
 
-	if (s->dbifcount > 0 && !cfg.alwaysadd) {
-		s->dbifcount = 0;
-		return;
-	}
-
 	if (debug) {
-		printf("db if count: %" PRIu64 "\n", s->dbifcount);
+		printf("initial db if count: %" PRIu64 "\n", s->dbifcount);
 	}
 
-	if (s->noadd) {
-		printf("No interfaces found in database, exiting.\n");
-		exit(EXIT_FAILURE);
+	if (s->dbifcount > 0 && !cfg.alwaysadd) {
+		return;
 	}
 
 	if (!spacecheck(cfg.dbdir)) {
@@ -319,24 +315,48 @@ void preparedatabase(DSTATE *s)
 		exit(EXIT_FAILURE);
 	}
 
+	if (s->initdb && s->noadd) {
+		return;
+	}
+
 	if (s->dbifcount == 0) {
-		if (importlegacydbs(s) && !cfg.alwaysadd) {
-			s->dbifcount = 0;
+		s->dbifcount += importlegacydbs();
+		if ((s->dbifcount > 0 && !cfg.alwaysadd) || s->initdb) {
 			return;
 		}
+	}
+
+	if (s->dbifcount == 0 && !cfg.alwaysadd) {
+		if (s->noadd && !s->startempty) {
+			printf("No interfaces found in database, exiting.\n");
+			exit(EXIT_FAILURE);
+		} else if (s->noadd && s->startempty) {
+			printf("Starting without interfaces to monitor.\n");
+			/* force counter sync for any later discovered interface */
+			s->sync = 1;
+			return;
+		}
+	}
+
+	if (s->dbifcount == 0) {
 		printf("No interfaces found in database, adding available interfaces...\n");
 	}
 
-	if (!addinterfaces(s) && s->dbifcount == 0) {
-		printf("Nothing to do, exiting.\n");
-		exit(EXIT_FAILURE);
-	}
+	s->dbifcount += addinterfaces(s);
 
-	/* set counter back to zero so that dbs will be cached later */
-	s->dbifcount = 0;
+	if (s->dbifcount == 0) {
+		if (!s->startempty) {
+			printf("No interfaces found for monitoring, exiting.\n");
+			exit(EXIT_FAILURE);
+		} else {
+			printf("Starting without interfaces to monitor.\n");
+			/* force counter sync for any later discovered interface */
+			s->sync = 1;
+		}
+	}
 }
 
-unsigned int importlegacydbs(DSTATE *s)
+unsigned int importlegacydbs(void)
 {
 	DIR *dir;
 	struct dirent *di;
@@ -349,9 +369,8 @@ unsigned int importlegacydbs(DSTATE *s)
 		exit(EXIT_FAILURE);
 	}
 
-	s->dbifcount = 0;
 	while ((di = readdir(dir))) {
-		if ((di->d_name[0] != '.') && (strncmp(di->d_name, DATABASEFILE, strlen(DATABASEFILE)) != 0)) {
+		if ((di->d_name[0] != '.') && (di->d_type != DT_DIR) && (strncmp(di->d_name, DATABASEFILE, strlen(DATABASEFILE)) != 0)) {
 			/* ignore already known interfaces */
 			if (db_getinterfacecountbyname(di->d_name)) {
 				continue;
@@ -363,7 +382,6 @@ unsigned int importlegacydbs(DSTATE *s)
 	}
 	closedir(dir);
 
-	s->dbifcount += importcount;
 	return importcount;
 }
 
@@ -410,7 +428,9 @@ void filldatabaselist(DSTATE *s)
 	}
 
 	iflistfree(&dbifl);
-	s->sync = 0;
+
+	/* force counter sync for any later discovered interface */
+	s->sync = 1;
 
 	/* disable update interval check for one loop if database list was refreshed */
 	/* otherwise increase default update interval since there's nothing else to do */
@@ -508,9 +528,9 @@ void processdatacache(DSTATE *s)
 
 int initcachevalues(const DSTATE *s, datacache **dc)
 {
-	interfaceinfo info;
+	interfaceinfo ifaceinfo;
 
-	if (!db_getinterfaceinfo((*dc)->interface, &info)) {
+	if (!db_getinterfaceinfo((*dc)->interface, &ifaceinfo)) {
 		return 0;
 	}
 
@@ -518,10 +538,10 @@ int initcachevalues(const DSTATE *s, datacache **dc)
 		(*dc)->currx = 0;
 		(*dc)->curtx = 0;
 	} else {
-		(*dc)->currx = info.rxcounter;
-		(*dc)->curtx = info.txcounter;
+		(*dc)->currx = ifaceinfo.rxcounter;
+		(*dc)->curtx = ifaceinfo.txcounter;
 	}
-	(*dc)->updated = info.updated;
+	(*dc)->updated = ifaceinfo.updated;
 	(*dc)->filled = 1;
 
 	return 1;
@@ -614,7 +634,7 @@ void flushcachetodisk(DSTATE *s)
 	uint32_t logcount = 0;
 	datacache *iterator = s->dcache;
 	xferlog *logiterator;
-	interfaceinfo info;
+	interfaceinfo ifaceinfo;
 
 	timeused(__func__, 1);
 
@@ -659,11 +679,11 @@ void flushcachetodisk(DSTATE *s)
 			}
 		}
 
-		if (!iterator->active && !logcount) {
+		if (!s->noremove && !iterator->active && !logcount) {
 			/* throw away if interface hasn't seen any data and is disabled */
 			if (!iterator->currx && !iterator->curtx) {
-				ret = db_getinterfaceinfo(iterator->interface, &info);
-				if (!ret || (!info.rxtotal && !info.txtotal)) {
+				ret = db_getinterfaceinfo(iterator->interface, &ifaceinfo);
+				if (!ret || (!ifaceinfo.rxtotal && !ifaceinfo.txtotal)) {
 					snprintf(errorstring, 1024, "Removing interface \"%s\" from database as it is disabled and has seen no data.", iterator->interface);
 					printe(PT_Info);
 					if (!db_removeinterface(iterator->interface)) {
@@ -825,7 +845,7 @@ void handleintsignals(DSTATE *s)
 			if (!db_open_rw(1)) {
 				snprintf(errorstring, 1024, "Opening database after SIGHUP failed (%s), exiting.", strerror(errno));
 				printe(PT_Error);
-				if (s->rundaemon && !debug) {
+				if (s->rundaemon == 1 && !debug) {
 					close(pidfile);
 					unlink(cfg.pidfile);
 				}
@@ -834,7 +854,7 @@ void handleintsignals(DSTATE *s)
 			if (!db_removedisabledresolutionentries()) {
 				snprintf(errorstring, 1024, "Disabled resolution entry cleanup after SIGHUP failed (%s), exiting.", strerror(errno));
 				printe(PT_Error);
-				if (s->rundaemon && !debug) {
+				if (s->rundaemon == 1 && !debug) {
 					close(pidfile);
 					unlink(cfg.pidfile);
 				}
@@ -844,7 +864,7 @@ void handleintsignals(DSTATE *s)
 				if (!db_vacuum()) {
 					snprintf(errorstring, 1024, "Database vacuum after SIGHUP failed (%s), exiting.", strerror(errno));
 					printe(PT_Error);
-					if (s->rundaemon && !debug) {
+					if (s->rundaemon == 1 && !debug) {
 						close(pidfile);
 						unlink(cfg.pidfile);
 					}
@@ -889,7 +909,7 @@ void preparedirs(const DSTATE *s)
 		updatedirowner(cfg.dbdir, s->user, s->group);
 	}
 
-	if (!cfg.createdirs || !s->rundaemon) {
+	if (!cfg.createdirs || s->rundaemon != 1) {
 		return;
 	}
 
@@ -1044,7 +1064,7 @@ __attribute__((noreturn)) void errorexitdaemon(DSTATE *s, const int fataldberror
 	datacache_clear(&s->dcache);
 	ibwflush();
 
-	if (s->rundaemon && !debug) {
+	if (s->rundaemon == 1 && !debug) {
 		close(pidfile);
 		unlink(cfg.pidfile);
 	}
