@@ -40,6 +40,8 @@ int db_open(const int createifnotfound, const int readonly)
 	if (stat(dbfilename, &filestat) != 0) {
 		if (errno == ENOENT && createifnotfound && !readonly) {
 			createdb = 1;
+			if (debug)
+				printf("\"%s\" doesn't exist, creating new database\n", dbfilename);
 		} else {
 			if (debug)
 				printf("Error (debug): Handling database \"%s\" failed: %s\n", dbfilename, strerror(errno));
@@ -49,6 +51,8 @@ int db_open(const int createifnotfound, const int readonly)
 		if (filestat.st_size == 0) {
 			if (createifnotfound) {
 				createdb = 1;
+				if (debug)
+					printf("\"%s\" exists but is 0 sized, creating new database\n", dbfilename);
 			} else {
 				printf("Error: Database \"%s\" contains 0 bytes and isn't a valid database, exiting.\n", dbfilename);
 				exit(EXIT_FAILURE);
@@ -1265,7 +1269,7 @@ int db_getdata(dbdatalist **dbdata, dbdatalistinfo *listinfo, const char *iface,
 int db_getdata_range(dbdatalist **dbdata, dbdatalistinfo *listinfo, const char *iface, const char *table, const uint32_t resultlimit, const char *databegin, const char *dataend)
 {
 	int ret, i, rc;
-	const char *datatables[] = {"fiveminute", "hour", "day", "month", "year", "top"};
+	const char *datatables[] = {"fiveminute", "hour", "day", "month", "year", "top", "percentile"};
 	char sql[768], limit[64], dbegin[37], dend[44], *ifaceidin = NULL;
 	sqlite3_stmt *sqlstmt;
 	time_t timestamp;
@@ -1275,7 +1279,7 @@ int db_getdata_range(dbdatalist **dbdata, dbdatalistinfo *listinfo, const char *
 	listinfo->count = 0;
 
 	ret = 0;
-	for (i = 0; i < 6; i++) {
+	for (i = 0; i < 7; i++) {
 		if (strcmp(table, datatables[i]) == 0) {
 			ret = 1;
 			break;
@@ -1325,6 +1329,13 @@ int db_getdata_range(dbdatalist **dbdata, dbdatalistinfo *listinfo, const char *
 		} else {
 			sqlite3_snprintf(768, sql, "select * from (select id, strftime('%%s', date, 'utc') as unixdate, sum(rx) as rx, sum(tx) as tx from top where interface in (%s) group by date order by rx+tx desc, unixdate asc %s) order by rx+tx asc, unixdate desc", ifaceidin, limit);
 		}
+	} else if (strcmp(table, "percentile") == 0) {
+		/* 'percentile' is a meta table collected from 'fiveminute' grouped as hours requiring as different query */
+		if (strlen(dbegin) && strlen(limit)) {
+			sqlite3_snprintf(768, sql, "select * from (select id, strftime('%%s', date, 'utc') as unixdate, sum(rx), sum(tx) from fiveminute where interface in (%s) %s %s group by strftime('%%Y-%%m-%%d %%H', date, 'utc') order by unixdate asc %s) order by unixdate desc", ifaceidin, dbegin, dend, limit);
+		} else {
+			sqlite3_snprintf(768, sql, "select id, strftime('%%s', date, 'utc') as unixdate, sum(rx), sum(tx) from fiveminute where interface in (%s) %s %s group by strftime('%%Y-%%m-%%d %%H', date, 'utc') order by unixdate desc %s", ifaceidin, dbegin, dend, limit);
+		}
 	} else {
 		if (strlen(dbegin) && strlen(limit)) {
 			sqlite3_snprintf(768, sql, "select * from (select id, strftime('%%s', date, 'utc') as unixdate, sum(rx), sum(tx) from %s where interface in (%s) %s %s group by date order by unixdate asc %s) order by unixdate desc", table, ifaceidin, dbegin, dend, limit);
