@@ -1,6 +1,7 @@
 #include "common.h"
 #include "dbsql.h"
 #include "misc.h"
+#include "percentile.h"
 #include "image.h"
 #include "image_support.h"
 #include "vnstati.h"
@@ -11,6 +12,7 @@ void initimagecontent(IMAGECONTENT *ic)
 	ic->font = gdFontGetSmall();
 	ic->lineheight = 12;
 	ic->large = 0;
+	ic->invert = 0;
 	ic->showheader = 1;
 	ic->showedge = 1;
 	ic->showlegend = 1;
@@ -58,6 +60,11 @@ void drawimage(IMAGECONTENT *ic)
 		case 10:
 			drawfivegraph(ic, cfg.hourlyrate, cfg.fivegresultcount, cfg.fivegheight);
 			break;
+		case 130:
+		case 131:
+		case 132:
+			draw95thpercentilegraph(ic, cfg.qmode - 130);
+			break;
 		default:
 			printf("Error: No such query mode: %d\n", cfg.qmode);
 			exit(EXIT_FAILURE);
@@ -105,7 +112,7 @@ void scaleimage(IMAGECONTENT *ic)
 }
 #endif
 
-int drawhours(IMAGECONTENT *ic, const int xpos, const int ypos, const int rate)
+int drawhours(IMAGECONTENT *ic, const int xpos, const int ypos, const int israte)
 {
 	int i, tmax = 0, s = 0, step, prev = 0, diff = 0, chour;
 	int x = xpos, y = ypos, extrax = 0, extray = 0, xt = 0;
@@ -169,7 +176,7 @@ int drawhours(IMAGECONTENT *ic, const int xpos, const int ypos, const int rate)
 
 	for (i = 0; i < 24; i++) {
 		/* convert hourly transfer to hourly rate if needed */
-		if (rate) {
+		if (israte) {
 			if ((ic->current - hourdata[i].date) > 3600) {
 				hourdata[i].rx = (uint64_t)((double)hourdata[i].rx / ratediv);
 				hourdata[i].tx = (uint64_t)((double)hourdata[i].tx / ratediv);
@@ -212,7 +219,7 @@ int drawhours(IMAGECONTENT *ic, const int xpos, const int ypos, const int rate)
 	}
 
 	/* scale values */
-	scaleunit = getscale(max, rate);
+	scaleunit = getscale(max, israte);
 
 	s = (int)lrint(((double)scaleunit / (double)max) * (124 + extray));
 	if (s < SCALEMINPIXELS) {
@@ -226,7 +233,7 @@ int drawhours(IMAGECONTENT *ic, const int xpos, const int ypos, const int rate)
 	for (i = step; i * s <= (124 + extray + 4); i = i + step) {
 		gdImageDashedLine(ic->im, xt, y + 124 - (i * s), xt + 424 + extrax, y + 124 - (i * s), ic->cline);
 		gdImageDashedLine(ic->im, xt, y + 124 - prev - (step * s) / 2, xt + 424 + extrax, y + 124 - prev - (step * s) / 2, ic->clinel);
-		gdImageString(ic->im, font, x + 16 - (ic->large * 3), y + 121 - (i * s) - (ic->large * 3), (unsigned char *)getimagevalue(scaleunit * (unsigned int)i, 3, rate), ic->ctext);
+		gdImageString(ic->im, font, x + 16 - (ic->large * 3), y + 121 - (i * s) - (ic->large * 3), (unsigned char *)getimagevalue(scaleunit * (unsigned int)i, 3, israte), ic->ctext);
 		prev = i * s;
 	}
 	if ((prev + (step * s) / 2) <= (124 + extray + 4)) {
@@ -234,7 +241,7 @@ int drawhours(IMAGECONTENT *ic, const int xpos, const int ypos, const int rate)
 	}
 
 	/* scale text */
-	gdImageStringUp(ic->im, font, x - 2 - (ic->large * 14), y + 58 + (rate * 10) - (extray / 2), (unsigned char *)getimagescale(scaleunit * (unsigned int)step, rate), ic->ctext);
+	gdImageStringUp(ic->im, font, x - 2 - (ic->large * 14), y + 58 + (israte * 10) - (extray / 2), (unsigned char *)getimagescale(scaleunit * (unsigned int)step, israte), ic->ctext);
 
 	/* axis */
 	gdImageLine(ic->im, xt - 4, y + 124, xt + 430 + extrax, y + 124, ic->ctext);
@@ -281,7 +288,7 @@ int drawhours(IMAGECONTENT *ic, const int xpos, const int ypos, const int rate)
 	return 1;
 }
 
-void drawhourly(IMAGECONTENT *ic, const int rate)
+void drawhourly(IMAGECONTENT *ic, const int israte)
 {
 	int width, height, headermod = 0;
 
@@ -296,7 +303,7 @@ void drawhourly(IMAGECONTENT *ic, const int rate)
 	imageinit(ic, width, height);
 	layoutinit(ic, " / hourly", width, height);
 
-	if (drawhours(ic, 12, 46 - headermod + (ic->large * 40), rate)) {
+	if (drawhours(ic, 12, 46 - headermod + (ic->large * 40), israte)) {
 		drawlegend(ic, width / 2 - (ic->large * 10), 183 - headermod + (ic->large * 46), 0);
 	}
 }
@@ -634,7 +641,7 @@ void drawlist(IMAGECONTENT *ic, const char *listname)
 	dbdatalistfree(&datalist);
 }
 
-void drawsummary(IMAGECONTENT *ic, const int layout, const int rate)
+void drawsummary(IMAGECONTENT *ic, const int layout, const int israte)
 {
 	int width, height, headermod;
 
@@ -676,17 +683,17 @@ void drawsummary(IMAGECONTENT *ic, const int layout, const int rate)
 		// horizontal
 		case 1:
 			if (cfg.summarygraph == 1) {
-				drawfiveminutes(ic, 496 + (ic->large * 174), height - 30 - (ic->large * 8), rate, 422 + (ic->large * 154), height - 68 + headermod - (ic->large * 8));
+				drawfiveminutes(ic, 496 + (ic->large * 174), height - 30 - (ic->large * 8), israte, 422 + (ic->large * 154), height - 68 + headermod - (ic->large * 8));
 			} else {
-				drawhours(ic, 500 + (ic->large * 160), 46 + (ic->large * 40) - headermod, rate);
+				drawhours(ic, 500 + (ic->large * 160), 46 + (ic->large * 40) - headermod, israte);
 			}
 			break;
 		// vertical
 		case 2:
 			if (cfg.summarygraph == 1) {
-				drawfiveminutes(ic, 8 + (ic->large * 14), height - 31 - (ic->large * 6), rate, 422 + (ic->large * 154), 132 + (ic->large * 35));
+				drawfiveminutes(ic, 8 + (ic->large * 14), height - 31 - (ic->large * 6), israte, 422 + (ic->large * 154), 132 + (ic->large * 35));
 			} else {
-				drawhours(ic, 12, 215 + (ic->large * 84) - headermod, rate);
+				drawhours(ic, 12, 215 + (ic->large * 84) - headermod, israte);
 			}
 			break;
 		default:
@@ -900,7 +907,7 @@ void drawsummary_digest(IMAGECONTENT *ic, const int x, const int y, const char *
 	dbdatalistfree(&datalist);
 }
 
-void drawfivegraph(IMAGECONTENT *ic, const int rate, const int resultcount, const int height)
+void drawfivegraph(IMAGECONTENT *ic, const int israte, const int resultcount, const int height)
 {
 	int imagewidth, imageheight = height, headermod = 0;
 
@@ -913,12 +920,12 @@ void drawfivegraph(IMAGECONTENT *ic, const int rate, const int resultcount, cons
 	imageinit(ic, imagewidth, imageheight);
 	layoutinit(ic, " / 5 minute", imagewidth, imageheight);
 
-	if (drawfiveminutes(ic, 8 + (ic->large * 14), imageheight - 30 - (ic->large * 8), rate, resultcount, imageheight - 68 + headermod - (ic->large * 8))) {
+	if (drawfiveminutes(ic, 8 + (ic->large * 14), imageheight - 30 - (ic->large * 8), israte, resultcount, imageheight - 68 + headermod - (ic->large * 8))) {
 		drawlegend(ic, imagewidth / 2 - (ic->large * 10), imageheight - 17 - (ic->large * 2), 0);
 	}
 }
 
-int drawfiveminutes(IMAGECONTENT *ic, const int xpos, const int ypos, const int rate, const int resultcount, const int height)
+int drawfiveminutes(IMAGECONTENT *ic, const int xpos, const int ypos, const int israte, const int resultcount, const int height)
 {
 	int x = xpos, y = ypos, i = 0, t = 0, rxh = 0, txh = 0, step = 0, s = 0, prev = 0;
 	uint64_t scaleunit, max;
@@ -985,7 +992,7 @@ int drawfiveminutes(IMAGECONTENT *ic, const int xpos, const int ypos, const int
 	gdImageString(ic->im, font, x - 21 - (ic->large * 3), y - 4 - (ic->large * 3), (unsigned char *)"  0", ic->ctext);
 
 	/* scale values */
-	scaleunit = getscale(max, rate);
+	scaleunit = getscale(max, israte);
 
 	s = (int)lrint(((double)scaleunit / (double)max) * t);
 	if (s == 0) {
@@ -1011,7 +1018,7 @@ int drawfiveminutes(IMAGECONTENT *ic, const int xpos, const int ypos, const int
 	for (i = step; i * s <= rxh; i = i + step) {
 		gdImageDashedLine(ic->im, x, y - (i * s), x + (resultcount + FIVEMINWIDTHPADDING), y - (i * s), ic->cline);
 		gdImageDashedLine(ic->im, x, y - prev - (step * s) / 2, x + (resultcount + FIVEMINWIDTHPADDING), y - prev - (step * s) / 2, ic->clinel);
-		gdImageString(ic->im, font, x - 21 - (ic->large * 3), y - 3 - (i * s) - (ic->large * 3), (unsigned char *)getimagevalue(scaleunit * (unsigned int)i, 3, rate), ic->ctext);
+		gdImageString(ic->im, font, x - 21 - (ic->large * 3), y - 3 - (i * s) - (ic->large * 3), (unsigned char *)getimagevalue(scaleunit * (unsigned int)i, 3, israte), ic->ctext);
 		prev = i * s;
 	}
 	if ((prev + (step * s) / 2) <= rxh) {
@@ -1025,7 +1032,7 @@ int drawfiveminutes(IMAGECONTENT *ic, const int xpos, const int ypos, const int
 	for (i = step; i * s <= txh; i = i + step) {
 		gdImageDashedLine(ic->im, x, y + (i * s), x + (resultcount + FIVEMINWIDTHPADDING), y + (i * s), ic->cline);
 		gdImageDashedLine(ic->im, x, y + prev + (step * s) / 2, x + (resultcount + FIVEMINWIDTHPADDING), y + prev + (step * s) / 2, ic->clinel);
-		gdImageString(ic->im, font, x - 21 - (ic->large * 3), y - 3 + (i * s) - (ic->large * 3), (unsigned char *)getimagevalue(scaleunit * (unsigned int)i, 3, rate), ic->ctext);
+		gdImageString(ic->im, font, x - 21 - (ic->large * 3), y - 3 + (i * s) - (ic->large * 3), (unsigned char *)getimagevalue(scaleunit * (unsigned int)i, 3, israte), ic->ctext);
 		prev = i * s;
 	}
 	if ((prev + (step * s) / 2) <= txh) {
@@ -1035,7 +1042,7 @@ int drawfiveminutes(IMAGECONTENT *ic, const int xpos, const int ypos, const int
 	y--; // y is now back on center line
 
 	/* scale text */
-	gdImageStringUp(ic->im, font, x - 39 - (ic->large * 14), ypos - height / 2 + (rate * 10), (unsigned char *)getimagescale(scaleunit * (unsigned int)step, rate), ic->ctext);
+	gdImageStringUp(ic->im, font, x - 39 - (ic->large * 14), ypos - height / 2 + (israte * 10), (unsigned char *)getimagescale(scaleunit * (unsigned int)step, israte), ic->ctext);
 
 	timestamp = datainfo.maxtime - (resultcount * 300);
 
@@ -1112,3 +1119,197 @@ int drawfiveminutes(IMAGECONTENT *ic, const int xpos, const int ypos, const int
 
 	return 1;
 }
+
+void draw95thpercentilegraph(IMAGECONTENT *ic, const int mode)
+{
+	int imagewidth, imageheight = 300, headermod = 0;
+
+	/* width needed for all percentile entries + decoration depending on font size */
+	imagewidth = PERCENTILEENTRYCOUNT + 78 + (ic->large * 14);
+
+	if (!ic->showheader) {
+		headermod = 22;
+	}
+
+	imageinit(ic, imagewidth, imageheight);
+	layoutinit(ic, " / 95th percentile", imagewidth, imageheight);
+
+	drawpercentile(ic, mode, 8 + (ic->large * 14), imageheight - 30 - (ic->large * 8), imageheight - 68 + headermod - (ic->large * 8));
+}
+
+void drawpercentile(IMAGECONTENT *ic, const int mode, const int xpos, const int ypos, const int height)
+{
+	int i, l, x = xpos, y = ypos, s = 0, step = 0, prev = 0, last = 0, color;
+	uint64_t scaleunit, max, percentile;
+	double ratediv, percentileratediv;
+	struct tm *d;
+	time_t current;
+	char datebuff[DATEBUFFLEN];
+	dbdatalist *datalist = NULL, *datalist_i = NULL;
+	dbdatalistinfo datainfo;
+	percentiledata pdata;
+	gdFontPtr font;
+
+	if (cfg.fiveminutehours < PERCENTILEENTRYCOUNT) {
+		fprintf(stderr, "\nWarning: Configuration \"5MinuteHours\" needs to be at least %d for 100%% coverage.\n", PERCENTILEENTRYCOUNT);
+		fprintf(stderr, "         \"5MinuteHours\" is currently set at %d.\n\n", cfg.fiveminutehours);
+	}
+
+	if (ic->large) {
+		font = gdFontGetSmall();
+	} else {
+		font = gdFontGetTiny();
+	}
+
+	if (!getpercentiledata(&pdata, ic->interface.name, 0)) {
+		gdImageString(ic->im, ic->font, x + 320 - (ic->large * 30), y - 120, (unsigned char *)"failed to get percentile data", ic->ctext);
+		return;
+	}
+
+	/* hourly/percentile bytes to rate */
+	if (cfg.rateunit) {
+		ratediv = 450;
+		percentileratediv = 37.5;
+	} else {
+		ratediv = 3600;
+		percentileratediv = 300;
+	}
+
+	d = localtime(&pdata.monthbegin);
+	strftime(datebuff, DATEBUFFLEN, "%Y-%m-%d", d);
+
+	if (!db_getdata_range(&datalist, &datainfo, ic->interface.name, "percentile", PERCENTILEENTRYCOUNT, datebuff, "") || datainfo.count == 0) {
+		gdImageString(ic->im, ic->font, x + 320 - (ic->large * 30), y - 120, (unsigned char *)"no percentile data available", ic->ctext);
+		return;
+	}
+
+	if (debug) {
+		printf("mode:  %d - %d\n", mode, cfg.qmode);
+		printf("count: %" PRIu32 "\n", datainfo.count);
+	}
+
+	if (mode == 0) {
+		color = ic->crx;
+		percentile = pdata.rxpercentile;
+		max = (uint64_t)((double)datainfo.maxrx / ratediv);
+	} else if (mode == 1) {
+		color = ic->ctx;
+		percentile = pdata.txpercentile;
+		max = (uint64_t)((double)datainfo.maxtx / ratediv);
+	} else {
+		color = ic->ctotal;
+		percentile = pdata.sumpercentile;
+		max = (uint64_t)((double)datainfo.max / ratediv);
+	}
+
+	if ((uint64_t)((double)(percentile) / percentileratediv) > max) {
+		max = (uint64_t)((double)(percentile) / percentileratediv);
+	}
+
+	/* scale values */
+	scaleunit = getscale(max, 1);
+
+	s = (int)lrint(((double)scaleunit / (double)max) * height);
+	if (s < SCALEMINPIXELS) {
+		step = 2;
+	} else {
+		step = 1;
+	}
+
+	/* scale text */
+	gdImageStringUp(ic->im, font, x - 2 - (ic->large * 14), y - (height / 2), (unsigned char *)getimagescale(scaleunit * (unsigned int)step, 1), ic->ctext);
+
+	/* axis */
+	x += 36;
+	gdImageLine(ic->im, x, y, x + (PERCENTILEENTRYCOUNT + PERCENTILEMINWIDTHFULLPADDING), y, ic->ctext);
+	gdImageLine(ic->im, x + 4, y + 4, x + 4, y - height, ic->ctext);
+
+	/* arrows */
+	drawarrowup(ic, x + 4, y - 4 - height);
+	drawarrowright(ic, x + 1 + (PERCENTILEENTRYCOUNT + PERCENTILEMINWIDTHFULLPADDING), y);
+
+	/* adjust cursor to first point on graph */
+	x += 5;
+	y -= 1;
+
+	for (i = step; i * s <= height; i = i + step) {
+		gdImageDashedLine(ic->im, x, y - (i * s), x + (PERCENTILEENTRYCOUNT + PERCENTILEMINWIDTHFULLPADDING) - 5, y - (i * s), ic->cline);
+		gdImageDashedLine(ic->im, x, y - prev - (step * s) / 2, x + (PERCENTILEENTRYCOUNT + PERCENTILEMINWIDTHFULLPADDING) - 5, y - prev - (step * s) / 2, ic->clinel);
+		gdImageString(ic->im, font, x - 22 - (ic->large * 3), y - 4 - (i * s) - (ic->large * 3), (unsigned char *)getimagevalue(scaleunit * (unsigned int)i, 3, 1), ic->ctext);
+		prev = i * s;
+	}
+	if ((prev + (step * s) / 2) <= height) {
+		gdImageDashedLine(ic->im, x, y - prev - (step * s) / 2, x + (PERCENTILEENTRYCOUNT + PERCENTILEMINWIDTHFULLPADDING) - 5, y - prev - (step * s) / 2, ic->clinel);
+	}
+
+	datalist_i = datalist;
+	current = pdata.monthbegin;
+	prev = -24;
+
+	/* draw data */
+	for (i = 0; i < PERCENTILEENTRYCOUNT; i++, current += 3600) {
+		if (datalist_i == NULL || current < datalist_i->timestamp) {
+			gdImageSetPixel(ic->im, x + i, y + 1, ic->cbgoffset);
+			if (i >= prev + 24 && i % 24 == 0 && current < pdata.dataend) {
+				d = localtime(&current);
+				strftime(datebuff, DATEBUFFLEN, "%d", d);
+				if (i > 0) {
+					drawpole(ic, x + i, y + 4, height, 1, ic->cbgoffset);
+				}
+				gdImageString(ic->im, font, x + 12 + i - 4 - (ic->large), y + 5, (unsigned char *)datebuff, ic->cline);
+				prev = i;
+			}
+			continue;
+		}
+
+		if (i >= prev + 24 && i % 24 == 0) {
+			d = localtime(&current);
+			strftime(datebuff, DATEBUFFLEN, "%d", d);
+			drawpole(ic, x + i, y, height, 1, ic->cbgoffset);
+			if (i > 0) {
+				gdImageLine(ic->im, x + i, y + 1, x + i, y + 4, ic->ctext);
+			}
+			gdImageString(ic->im, font, x + 12 + i - 4 - (ic->large), y + 5, (unsigned char *)datebuff, ic->ctext);
+			prev = i;
+		}
+
+		if (mode == 0) {
+			l = (int)lrint(((double)(datalist_i->rx) / (double)ratediv / (double)max) * height);
+		} else if (mode == 1) {
+			l = (int)lrint(((double)(datalist_i->tx) / (double)ratediv / (double)max) * height);
+		} else {
+			l = (int)lrint(((double)(datalist_i->rx + datalist_i->tx) / (double)ratediv / (double)max) * height);
+		}
+		if (l > height) {
+			l = height;
+		}
+		drawpole(ic, x + i, y, l, 1, color);
+
+		last = i;
+		datalist_i = datalist_i->next;
+	}
+
+	dbdatalistfree(&datalist);
+
+	/* 95th percentile line */
+	l = (int)lrint(((double)(percentile) / percentileratediv / (double)max) * height);
+	if (l > height) {
+		l = height;
+	} else if (l == 0) {
+		l = 1;
+	}
+	gdImageLine(ic->im, x, y - l, x + last, y - l, ic->cpercentileline);
+
+	if (debug) {
+		printf("s:   %d\n", s);
+		printf("l:   %d\n", l);
+		printf("h:   %d\n", height);
+		printf("p:   %" PRIu64 "\n", (uint64_t)(percentile / percentileratediv));
+		printf("max: %" PRIu64 "\n", max);
+		printf("max rate: %s\n", gettrafficrate(max * ratediv, 3600, 0));
+		printf("per rate: %s\n", gettrafficrate(percentile, 300, 0));
+	}
+
+	/* finally add legend with percentile text */
+	drawpercentilelegend(ic, x + 300 - (ic->large * 50), y + 14 + (ic->large * 6), mode, percentile);
+}
