@@ -7,78 +7,106 @@
 
 void imageinit(IMAGECONTENT *ic, const int width, const int height)
 {
-	int rgb[3];
+	int rgb[3], invert = 1;
 
 	ic->im = gdImageCreate(width, height);
 
+	if (ic->invert > 0) {
+		invert = -1;
+	}
+
 	/* text, edge and header colors */
 	hextorgb(cfg.ctext, rgb);
+	if (ic->invert > 0) { invertcolor(rgb); }
 	ic->ctext = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("ctext", ic->ctext, cfg.ctext, rgb);
 	hextorgb(cfg.cedge, rgb);
+	if (ic->invert > 0) { invertcolor(rgb); }
 	ic->cedge = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("cedge", ic->cedge, cfg.cedge, rgb);
 	hextorgb(cfg.cheader, rgb);
+	if (ic->invert > 0) { invertcolor(rgb); }
 	ic->cheader = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("cheader", ic->cheader, cfg.cheader, rgb);
 	hextorgb(cfg.cheadertitle, rgb);
+	if (ic->invert > 0) { invertcolor(rgb); }
 	ic->cheadertitle = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("cheadertitle", ic->cheadertitle, cfg.cheadertitle, rgb);
 	hextorgb(cfg.cheaderdate, rgb);
+	if (ic->invert > 0) { invertcolor(rgb); }
 	ic->cheaderdate = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("cheaderdate", ic->cheaderdate, cfg.cheaderdate, rgb);
 
 	/* lines */
 	hextorgb(cfg.cline, rgb);
+	if (ic->invert > 0) { invertcolor(rgb); }
 	ic->cline = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("cline", ic->cline, cfg.cline, rgb);
 	if (cfg.clinel[0] == '-') {
-		modcolor(rgb, 50, 1);
+		modcolor(rgb, 50 * invert, 1);
 	} else {
 		hextorgb(cfg.clinel, rgb);
+		if (ic->invert > 0) { invertcolor(rgb); }
 	}
 	ic->clinel = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("clinel", ic->clinel, cfg.clinel, rgb);
+	hextorgb(cfg.cpercentileline, rgb);
+	if (ic->invert > 0) { invertcolor(rgb); }
+	ic->cpercentileline = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
+	colorinitcheck("cpercentileline", ic->cpercentileline, cfg.cpercentileline, rgb);
 
 	/* background */
 	hextorgb(cfg.cbg, rgb);
+	if (ic->invert > 0) { invertcolor(rgb); }
 	ic->cbackground = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("cbackground", ic->cbackground, cfg.cbg, rgb);
-	modcolor(rgb, -35, 0);
+	modcolor(rgb, -35 * invert, 0);
 	ic->cvnstat = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("cvnstat", ic->cvnstat, cfg.cbg, rgb);
 	hextorgb(cfg.cbg, rgb);
-	modcolor(rgb, -15, 0);
+	if (ic->invert > 0) { invertcolor(rgb); }
+	modcolor(rgb, -15 * invert, 0);
 	ic->cbgoffset = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("cbgoffset", ic->cbgoffset, cfg.cbg, rgb);
 	hextorgb(cfg.cbg, rgb);
-	modcolor(rgb, -40, 0);
+	if (ic->invert > 0) { invertcolor(rgb); }
+	modcolor(rgb, -40 * invert, 0);
 	ic->cbgoffsetmore = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("cbgoffsetmore", ic->cbgoffsetmore, cfg.cbg, rgb);
 
 	/* rx */
 	hextorgb(cfg.crx, rgb);
+	if (ic->invert > 1) { invertcolor(rgb); }
 	ic->crx = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("crx", ic->crx, cfg.crx, rgb);
 	if (cfg.crxd[0] == '-') {
-		modcolor(rgb, -50, 1);
+		modcolor(rgb, -50 * invert, 1);
 	} else {
 		hextorgb(cfg.crxd, rgb);
+		if (ic->invert > 1) { invertcolor(rgb); }
 	}
 	ic->crxd = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("crxd", ic->crxd, cfg.crxd, rgb);
 
 	/* tx */
 	hextorgb(cfg.ctx, rgb);
+	if (ic->invert > 1) { invertcolor(rgb); }
 	ic->ctx = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("ctx", ic->ctx, cfg.ctx, rgb);
 	if (cfg.ctxd[0] == '-') {
-		modcolor(rgb, -50, 1);
+		modcolor(rgb, -50 * invert, 1);
 	} else {
 		hextorgb(cfg.ctxd, rgb);
+		if (ic->invert > 1) { invertcolor(rgb); }
 	}
 	ic->ctxd = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
 	colorinitcheck("ctxd", ic->ctxd, cfg.ctxd, rgb);
+
+	/* total */
+	hextorgb(cfg.ctotal, rgb);
+	if (ic->invert > 1) { invertcolor(rgb); }
+	ic->ctotal = gdImageColorAllocate(ic->im, rgb[0], rgb[1], rgb[2]);
+	colorinitcheck("ctotal", ic->ctotal, cfg.ctotal, rgb);
 }
 
 void colorinitcheck(const char *color, const int value, const char *cfgtext, const int *rgb)
@@ -165,6 +193,33 @@ void drawlegend(IMAGECONTENT *ic, const int x, const int y, const short israte)
 	}
 }
 
+void drawpercentilelegend(IMAGECONTENT *ic, const int x, const int y, const int mode, const uint64_t percentile)
+{
+	int color, xoffset = 0;
+	char modetext[6], percentiletext[64];
+
+	if (mode == 0) {
+		snprintf(modetext, 6, "rx");
+		color = ic->crx;
+	} else if (mode == 1) {
+		snprintf(modetext, 6, "tx");
+		color = ic->ctx;
+	} else {
+		snprintf(modetext, 6, "total");
+		color = ic->ctotal;
+		xoffset = 18 + (ic->large * 6);
+	}
+
+	snprintf(percentiletext, 64, "%s     95th percentile: %s", modetext, gettrafficrate(percentile, 300, 0));
+	gdImageString(ic->im, ic->font, x, y, (unsigned char *)percentiletext, ic->ctext);
+
+	gdImageFilledRectangle(ic->im, x - 12 - (ic->large * 2), y + 4, x - 12 + ic->font->w - (ic->large * 2), y + 4 + ic->font->w, color);
+	gdImageRectangle(ic->im, x - 12 - (ic->large * 2), y + 4, x - 12 + ic->font->w - (ic->large * 2), y + 4 + ic->font->w, ic->ctext);
+
+	gdImageFilledRectangle(ic->im, x + 30 + (ic->large * 12) + xoffset, y + 4, x + 30 + ic->font->w + (ic->large * 12) + xoffset, y + 4 + ic->font->w, ic->cpercentileline);
+	gdImageRectangle(ic->im, x + 30 + (ic->large * 12) + xoffset, y + 4, x + 30 + ic->font->w + (ic->large * 12) + xoffset, y + 4 + ic->font->w, ic->ctext);
+}
+
 void drawbar(IMAGECONTENT *ic, const int x, const int y, const int len, const uint64_t rx, const uint64_t tx, const uint64_t max, const short isestimate)
 {
 	int rxl, txl, width = len, overlap = 0;
@@ -352,6 +407,7 @@ void drawarrowup(IMAGECONTENT *ic, const int x, const int y)
 	gdImageLine(ic->im, x, y, x - 2, y + 3, ic->ctext);
 	gdImageLine(ic->im, x - 2, y + 3, x + 2, y + 3, ic->ctext);
 	gdImageLine(ic->im, x, y + 1, x, y - 1, ic->ctext);
+	gdImageLine(ic->im, x, y, x, y + 2, ic->ctext);
 }
 
 void drawarrowright(IMAGECONTENT *ic, const int x, const int y)
@@ -429,7 +485,24 @@ void modcolor(int *rgb, const int offset, const int force)
 	}
 }
 
-char *getimagevalue(const uint64_t b, const int len, const int rate)
+void invertcolor(int *rgb)
+{
+	int i;
+
+	if (debug) {
+		printf("invert: %d, %d, %d -> ", rgb[0], rgb[1], rgb[2]);
+	}
+
+	for (i = 0; i < 3; i++) {
+		rgb[i] = 255 - rgb[i];
+	}
+
+	if (debug) {
+		printf("%d, %d, %d\n", rgb[0], rgb[1], rgb[2]);
+	}
+}
+
+char *getimagevalue(const uint64_t b, const int len, const int israte)
 {
 	static char buffer[64];
 	int i, declen = 0, unit = 0, p = 1024;
@@ -438,7 +511,7 @@ char *getimagevalue(const uint64_t b, const int len, const int rate)
 	if (b == 0) {
 		snprintf(buffer, 64, "%*s", len, "--");
 	} else {
-		if (rate && (getunit() == 2 || getunit() == 4)) {
+		if (israte && (getunit() == 2 || getunit() == 4)) {
 			p = 1000;
 			unit = getunit();
 		}
@@ -455,7 +528,7 @@ char *getimagevalue(const uint64_t b, const int len, const int rate)
 	return buffer;
 }
 
-char *getimagescale(const uint64_t b, const int rate)
+char *getimagescale(const uint64_t b, const int israte)
 {
 	static char buffer[8];
 	int unit, div = 1, p = 1024;
@@ -465,7 +538,7 @@ char *getimagescale(const uint64_t b, const int rate)
 	if (b == 0) {
 		snprintf(buffer, 8, "--");
 	} else {
-		if (rate) {
+		if (israte) {
 			if (unit == 2 || unit == 4) {
 				p = 1000;
 			}
@@ -483,7 +556,7 @@ char *getimagescale(const uint64_t b, const int rate)
 	return buffer;
 }
 
-uint64_t getscale(const uint64_t input, const int rate)
+uint64_t getscale(const uint64_t input, const int israte)
 {
 	int i, unit;
 	unsigned int div = 1024;
@@ -491,7 +564,7 @@ uint64_t getscale(const uint64_t input, const int rate)
 
 	unit = getunit();
 
-	if (rate && (unit == 2 || unit == 4)) {
+	if (israte && (unit == 2 || unit == 4)) {
 		div = 1000;
 	}
 
