@@ -49,6 +49,8 @@ void initparams(PARAMS *p)
 	p->alerttype = 0;
 	p->alertcondition = 0;
 	p->alertlimit = 0;
+	p->alertrateunit = -1;
+	p->alertrateunitmode = -1;
 
 	/* load default config */
 	defaultcfg();
@@ -69,6 +71,7 @@ void showhelp(const PARAMS *p)
 	printf("      -b, --begin <date>           set list begin date\n");
 	printf("      -e, --end <date>             set list end date\n\n");
 
+	printf("      --95th                       show 95th percentile\n");
 	printf("      --oneline [mode]             show simple parsable format\n");
 	printf("      --json [mode] [limit]        show database in json format\n");
 	printf("      --xml [mode] [limit]         show database in xml format\n");
@@ -92,7 +95,7 @@ void showlonghelp(const PARAMS *p)
 
 	printf("Query:\n");
 
-	printf("      -q,  --query                 query database\n");
+	printf("      -q,  --query [mode]          query database\n");
 	printf("      -s,  --short                 use short output\n");
 	printf("      -5,  --fiveminutes [limit]   show 5 minutes\n");
 	printf("      -h,  --hours [limit]         show hours\n");
@@ -243,6 +246,22 @@ void parseargs(PARAMS *p, const int argc, char **argv)
 			exit(EXIT_FAILURE);
 		} else if ((strcmp(argv[currentarg], "-q") == 0) || (strcmp(argv[currentarg], "--query") == 0)) {
 			p->query = 1;
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
+				if (argv[currentarg + 1][0] == 'a') {
+					cfg.qmode = 0;
+				} else if (argv[currentarg + 1][0] == 's') {
+					cfg.qmode = 4;
+				} else {
+					if (!ishelprequest(argv[currentarg + 1]))
+						printf("Error: Invalid mode parameter \"%s\".\n", argv[currentarg + 1]);
+					printf(" Valid parameters for %s:\n", argv[currentarg]);
+					printf("    (none) - use QueryMode configuration\n");
+					printf("    a      - summary for all interfaces\n");
+					printf("    s      - summary for single interface\n");
+					exit(EXIT_FAILURE);
+				}
+				currentarg++;
+			}
 		} else if ((strcmp(argv[currentarg], "-D") == 0) || (strcmp(argv[currentarg], "--debug") == 0)) {
 			debug = 1;
 		} else if ((strcmp(argv[currentarg], "-d") == 0) || (strcmp(argv[currentarg], "--days") == 0)) {
@@ -309,6 +328,8 @@ void parseargs(PARAMS *p, const int argc, char **argv)
 				}
 				currentarg++;
 			}
+		} else if ((strcmp(argv[currentarg], "--95th") == 0) || (strcmp(argv[currentarg], "--95") == 0) || (strcmp(argv[currentarg], "--95%") == 0)) {
+			cfg.qmode = 13;
 		} else if (strcmp(argv[currentarg], "--oneline") == 0) {
 			cfg.qmode = 9;
 			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
@@ -328,11 +349,11 @@ void parseargs(PARAMS *p, const int argc, char **argv)
 			cfg.qmode = 8;
 			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
 				p->xmlmode = argv[currentarg + 1][0];
-				if (strlen(argv[currentarg + 1]) != 1 || strchr("asfhdmyt", p->xmlmode) == NULL) {
+				if (strlen(argv[currentarg + 1]) != 1 || strchr("asfhdmytp", p->xmlmode) == NULL) {
 					if (!ishelprequest(argv[currentarg + 1]))
 						printf("Error: Invalid mode parameter \"%s\".\n", argv[currentarg + 1]);
 					printf(" Valid parameters for --xml:\n");
-					printf("    a - all (default)\n");
+					printf("    a - all except 95th percentile (default)\n");
 					printf("    s - summary\n");
 					printf("    f - only 5 minutes\n");
 					printf("    h - only hours\n");
@@ -340,6 +361,7 @@ void parseargs(PARAMS *p, const int argc, char **argv)
 					printf("    m - only months\n");
 					printf("    y - only years\n");
 					printf("    t - only top\n");
+					printf("    p - 95th percentile\n");
 					exit(EXIT_FAILURE);
 				}
 				if (p->xmlmode == 's') {
@@ -359,11 +381,11 @@ void parseargs(PARAMS *p, const int argc, char **argv)
 			cfg.qmode = 10;
 			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
 				p->jsonmode = argv[currentarg + 1][0];
-				if (strlen(argv[currentarg + 1]) != 1 || strchr("asfhdmyt", p->jsonmode) == NULL) {
+				if (strlen(argv[currentarg + 1]) != 1 || strchr("asfhdmytp", p->jsonmode) == NULL) {
 					if (!ishelprequest(argv[currentarg + 1]))
 						printf("Error: Invalid mode parameter \"%s\".\n", argv[currentarg + 1]);
 					printf(" Valid parameters for --json:\n");
-					printf("    a - all (default)\n");
+					printf("    a - all except 95th percentile (default)\n");
 					printf("    s - summary\n");
 					printf("    f - only 5 minutes\n");
 					printf("    h - only hours\n");
@@ -371,6 +393,7 @@ void parseargs(PARAMS *p, const int argc, char **argv)
 					printf("    m - only months\n");
 					printf("    y - only years\n");
 					printf("    t - only top\n");
+					printf("    p - 95th percentile\n");
 					exit(EXIT_FAILURE);
 				}
 				if (p->jsonmode == 's') {
@@ -478,12 +501,13 @@ void parseargs(PARAMS *p, const int argc, char **argv)
 		} else if (strcmp(argv[currentarg], "--iflist") == 0) {
 			p->query = 0;
 			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
-				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
+				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 2 || atoi(argv[currentarg + 1]) < 0) {
 					if (!ishelprequest(argv[currentarg + 1]))
 						printf("Error: Invalid mode parameter \"%s\".\n", argv[currentarg + 1]);
 					printf(" Valid parameters for --iflist:\n");
 					printf("    0 - show verbose (default)\n");
 					printf("    1 - show one interface per line\n");
+					printf("    2 - show only interface count\n");
 					exit(EXIT_FAILURE);
 				}
 				p->query = atoi(argv[currentarg + 1]);
@@ -494,12 +518,13 @@ void parseargs(PARAMS *p, const int argc, char **argv)
 		} else if (strcmp(argv[currentarg], "--dbiflist") == 0) {
 			p->query = 0;
 			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
-				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
+				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 2 || atoi(argv[currentarg + 1]) < 0) {
 					if (!ishelprequest(argv[currentarg + 1]))
 						printf("Error: Invalid mode parameter \"%s\".\n", argv[currentarg + 1]);
 					printf(" Valid parameters for --dbiflist:\n");
 					printf("    0 - show verbose (default)\n");
 					printf("    1 - show one interface per line\n");
+					printf("    2 - show only interface count\n");
 					exit(EXIT_FAILURE);
 				}
 				p->query = atoi(argv[currentarg + 1]);
@@ -572,7 +597,7 @@ int parsealertargs(PARAMS *p, char **argv)
 	int i, u, found, currentarg = 0;
 	uint64_t alertlimit = 0, unitmultiplier = 1;
 	int32_t unitmode = cfg.unitmode;
-	const char *alerttypes[] = {"h", "hour", "hourly", "d", "day", "daily", "m", "month", "monthly", "y", "year", "yearly"};
+	const char *alerttypes[] = {"h", "hour", "hourly", "d", "day", "daily", "m", "month", "monthly", "y", "year", "yearly", "p", "95", "95%"};
 	const char *alertconditions[] = {"rx", "tx", "total", "rx_estimate", "tx_estimate", "total_estimate"}; // order must match that of AlertCondition in dbshow.h
 
 	for (i = 1; i <= 6; i++) {
@@ -625,7 +650,7 @@ int parsealertargs(PARAMS *p, char **argv)
 
 	// type
 	found = 0;
-	for (i = 0; i < 12; i++) {
+	for (i = 0; i < 15; i++) {
 		if (strcmp(argv[currentarg], alerttypes[i]) == 0) {
 			found = 1;
 			break;
@@ -650,6 +675,10 @@ int parsealertargs(PARAMS *p, char **argv)
 		case 'y':
 			p->alerttype = AT_Year;
 			break;
+		case 'p':
+		case '9':
+			p->alerttype = AT_Percentile;
+			break;
 		default:
 			return 0;
 	}
@@ -658,6 +687,12 @@ int parsealertargs(PARAMS *p, char **argv)
 	}
 	currentarg++;
 
+	if (p->alerttype == AT_Percentile && (p->alertoutput == 2 || p->alertexit == 2)) {
+		printf("Error: Estimate output and exit conditions cannot be used in combination with selected alert type.\n");
+		showalerthelp();
+		return 0;
+	}
+
 	// condition
 	found = 0;
 	for (i = 0; i < 3 + (cfg.estimatevisible * 3); i++) {
@@ -689,6 +724,12 @@ int parsealertargs(PARAMS *p, char **argv)
 		return 0;
 	}
 
+	if (p->alerttype == AT_Percentile && (p->alertcondition == AC_RX_Estimate || p->alertcondition == AC_TX_Estimate || p->alertcondition == AC_Total_Estimate)) {
+		printf("Error: Estimate conditions cannot be used in combination with selected alert type.\n");
+		showalerthelp();
+		return 0;
+	}
+
 	// limit
 	if (!isnumeric(argv[currentarg])) {
 		printf("Error: Limit parameter for %s must be a greater than zero integer without decimals.\n", argv[0]);
@@ -708,19 +749,54 @@ int parsealertargs(PARAMS *p, char **argv)
 
 	// limit unit
 	found = 0;
-	for (u = 0; u < 3; u++) {
-		cfg.unitmode = u;
-		for (i = 1; i <= UNITPREFIXCOUNT; i++) {
-			if (strcmp(argv[currentarg], getunitprefix(i)) == 0) {
-				found = 1;
+	if (p->alerttype != AT_Percentile) {
+		for (u = 0; u < 3; u++) {
+			cfg.unitmode = u;
+			for (i = 1; i <= UNITPREFIXCOUNT; i++) {
+				if (strcmp(argv[currentarg], getunitprefix(i)) == 0) {
+					found = 1;
+					break;
+				}
+			}
+			if (found) {
+				break;
+			}
+		}
+		cfg.unitmode = unitmode;
+	} else {
+		for (u = 0; u < 5; u++) {
+			if (u == 1) {
+				continue;
+			}
+			for (i = 1; i <= UNITPREFIXCOUNT; i++) {
+				if (strcmp(argv[currentarg], getrateunitprefix(u, i)) == 0) {
+					found = 1;
+					break;
+				}
+			}
+			if (found) {
 				break;
 			}
 		}
 		if (found) {
-			break;
+			// override configuration to use same units as user gave as parameter
+			if (u > 2) {
+				cfg.rateunit = 1;
+				p->alertrateunit = 1;
+			} else {
+				cfg.rateunit = 0;
+				p->alertrateunit = 0;
+			}
+			if (u == 0 || u == 3) {
+				cfg.rateunitmode = 0;
+				p->alertrateunitmode = 0;
+			} else {
+				cfg.rateunitmode = 1;
+				p->alertrateunitmode = 1;
+			}
 		}
+
 	}
-	cfg.unitmode = unitmode;
 	if (!found) {
 		printf("Error: Invalid limit unit parameter \"%s\" for %s.\n", argv[currentarg], argv[0]);
 		showalerthelp();
@@ -736,6 +812,14 @@ int parsealertargs(PARAMS *p, char **argv)
 
 	p->alertlimit = alertlimit * unitmultiplier;
 
+	if (p->alerttype == AT_Percentile && cfg.rateunit == 1) {
+		if (p->alertlimit < 8) {
+			printf("Error: Limit needs to be at least 8 bits per second.\n");
+			return 0;
+		}
+		p->alertlimit /= 8;
+	}
+
 	if (debug) {
 		printf("Alert unit %s is %d %d = %" PRIu64 "\n", argv[currentarg], u, i, unitmultiplier);
 		printf("Alert real limit is %" PRIu64 " * %" PRIu64 " = %" PRIu64 "\n", alertlimit, unitmultiplier, p->alertlimit);
@@ -762,7 +846,7 @@ void showalerthelp(void)
 	printf("    3 - use exit status 1 if limit is exceeded\n\n");
 
 	printf(" <type>\n");
-	printf("    h, hour, hourly        d, day, daily\n");
+	printf("    h, hour, hourly        d, day, daily        p, 95, 95%%\n");
 	printf("    m, month, monthly      y, year, yearly\n\n");
 
 	printf(" <condition>\n");
@@ -773,11 +857,17 @@ void showalerthelp(void)
 	}
 
 	printf(" <limit>\n");
-	printf("    greater than zero integer without decimals\n\n");
+	printf("    greater than zero integer, no decimals\n\n");
 
 	printf(" <unit> for <limit>\n");
 	printf("    B, KiB, MiB, GiB, TiB, PiB, EiB\n");
-	printf("    B, KB, MB, GB, TB, PB, EB\n");
+	printf("    B, KB, MB, GB, TB, PB, EB\n\n");
+
+	printf(" <unit> for <limit> when 95th percentile <type> is used\n");
+	printf("    B/s, KiB/s, MiB/s, GiB/s, TiB/s, PiB/s, EiB/s                (IEC 1024^n)\n");
+	printf("    B/s, kB/s, MB/s, GB/s, TB/s, PB/s, EB/s                      (SI  1000^n)\n");
+	printf("    bit/s, Kibit/s, Mibit/s, Gibit/s, Tibit/s, Pibit/s, Eibit/s  (IEC 1024^n)\n");
+	printf("    bit/s, kbit/s, Mbit/s, Gbit/s, Tbit/s, Pbit/s, Ebit/s        (SI  1000^n)\n");
 }
 
 void showstylehelp(void)
@@ -806,6 +896,11 @@ void handleshowalert(PARAMS *p)
 
 	validateinterface(p);
 
+	if (p->alertrateunit > -1 && p->alertrateunitmode > -1) {
+		cfg.rateunit = p->alertrateunit;
+		cfg.rateunitmode = p->alertrateunitmode;
+	}
+
 	alert = showalert(p->interface, p->alertoutput, p->alertexit, p->alerttype, p->alertcondition, p->alertlimit);
 
 	if ((alert || p->alertexit == AE_Always_Exit_1) && p->alertexit != AE_Always_Exit_0) {
@@ -1030,7 +1125,7 @@ void handleshowdata(PARAMS *p)
 		} else if (cfg.qmode == 8) {
 			xmlheader();
 		} else if (cfg.qmode == 10) {
-			jsonheader();
+			jsonheader(JSONVERSION);
 		}
 
 		if (db_getiflist(&dbifl) <= 0) {
@@ -1089,7 +1184,7 @@ void showoneinterface(PARAMS *p)
 		showxml(p->interface, p->xmlmode, p->databegin, p->dataend);
 		xmlfooter();
 	} else if (cfg.qmode == 10) {
-		jsonheader();
+		jsonheader(JSONVERSION);
 		showjson(p->interface, 0, p->jsonmode, p->databegin, p->dataend);
 		jsonfooter();
 	}
@@ -1209,8 +1304,13 @@ void handleifselection(PARAMS *p)
 
 		iflistfree(&ifl);
 	} else if (p->query) {
-		if (db_getiflist_sorted(&dbifl, 1) <= 0) {
-			printf("Error: Unable to discover suitable interface from database.\n");
+		dbifcount = db_getiflist_sorted(&dbifl, 1);
+		if (dbifcount <= 0) {
+			if (dbifcount == 0) {
+				printf("Error: Database is empty.\n");
+			} else {
+				printf("Error: Unable to discover suitable interface from database.\n");
+			}
 			iflistfree(&dbifl);
 			exit(EXIT_FAILURE);
 		}
@@ -1222,12 +1322,13 @@ void handleifselection(PARAMS *p)
 	iflistfree(&dbifl);
 }
 
-void showiflist(const int parseable)
+void showiflist(const int mode)
 {
+	int ifcount = 0;
 	char *ifacelist = NULL;
 	iflist *ifl = NULL, *ifl_iterator = NULL;
 
-	if (!parseable) {
+	if (mode == 0) {
 		if (!getifliststring(&ifacelist, 1)) {
 			exit(EXIT_FAILURE);
 		}
@@ -1243,14 +1344,20 @@ void showiflist(const int parseable)
 		}
 		ifl_iterator = ifl;
 		while (ifl_iterator != NULL) {
-			printf("%s\n", ifl_iterator->interface);
+			if (mode != 2) {
+				printf("%s\n", ifl_iterator->interface);
+			}
+			ifcount++;
 			ifl_iterator = ifl_iterator->next;
 		}
 		iflistfree(&ifl);
+		if (mode == 2) {
+			printf("%d\n", ifcount);
+		}
 	}
 }
 
-void showdbiflist(const int parseable)
+void showdbiflist(const int mode)
 {
 	int dbifcount;
 	iflist *dbifl = NULL, *dbifl_i = NULL;
@@ -1260,18 +1367,25 @@ void showdbiflist(const int parseable)
 		exit(EXIT_FAILURE);
 	}
 
+	if (mode == 2) {
+		dbifcount = db_getinterfacecount();
+		printf("%d\n", dbifcount);
+		db_close();
+		return;
+	}
+
 	dbifcount = db_getiflist(&dbifl);
 	if (dbifcount < 0) {
 		printf("Error: Failed to get interface list from database \"%s/%s\".\n", cfg.dbdir, DATABASEFILE);
 		exit(EXIT_FAILURE);
 	}
 
-	if (dbifcount == 0 && !parseable) {
-		printf("Database is empty.");
+	if (dbifcount == 0 && mode == 0) {
+		printf("Database is empty.\n");
 	} else {
 		dbifl_i = dbifl;
 
-		if (!parseable) {
+		if (mode == 0) {
 			printf("Interfaces in database:");
 			while (dbifl_i != NULL) {
 				printf(" %s", dbifl_i->interface);
