@@ -11,6 +11,7 @@ typedef struct {
 	char definterface[MAXIFPARAMLEN], cfgfile[512], *ifacelist, jsonmode, xmlmode;
 	char databegin[18], dataend[18];
 	unsigned int alert, alertoutput, alertexit, alerttype, alertcondition;
+	int alertrateunit, alertrateunitmode;
 	uint64_t alertlimit;
 } PARAMS;
 
@@ -30,8 +31,8 @@ void handleshowdata(PARAMS *p);
 void showoneinterface(PARAMS *p);
 void handletrafficmeters(PARAMS *p);
 void handleifselection(PARAMS *p);
-void showiflist(const int parseable);
-void showdbiflist(const int parseable);
+void showiflist(const int mode);
+void showdbiflist(const int mode);
 void validateinterface(PARAMS *p);
 
 #endif
