@@ -1,5 +1,5 @@
 /*
-vnStat image output - Copyright (C) 2007-2023 Teemu Toivola <tst@iki.fi>
+vnStat image output - Copyright (C) 2007-2024 Teemu Toivola <tst@iki.fi>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -127,12 +127,14 @@ void showihelp(const IPARAMS *p)
 	printf("      -t,  --top [limit]                 output top days\n");
 	printf("      -s,  --summary                     output summary\n");
 	printf("      -hs, --hsummary [graph]            output horizontal summary with graph\n");
-	printf("      -vs, --vsummary [graph]            output vertical summary with graph\n\n");
+	printf("      -vs, --vsummary [graph]            output vertical summary with graph\n");
+	printf("      --95th <mode>                      output 95th percentile graph\n\n");
 
 	printf("      -nh, --noheader                    remove header from output\n");
 	printf("      -ne, --noedge                      remove edge from output\n");
 	printf("      -nl, --nolegend                    remove legend from output\n");
 	printf("      -ru, --rateunit [mode]             swap configured rate unit\n");
+	printf("      -ic, --invert-colors [mode]        invert image colors\n\n");
 	printf("      -S,  --small                       use small fonts");
 	if (!cfg.largefonts) {
 		printf(" (default)");
@@ -378,9 +380,9 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
 					if (!ishelprequest(argv[currentarg + 1]))
 						fprintf(stderr, "Error: Invalid mode selection \"%s\".\n", argv[currentarg + 1]);
-					printf(" Valid modes for %s:\n", argv[currentarg]);
-					printf("    0 - 24 hour sliding windows\n");
-					printf("    1 - graph start from midnight\n");
+					fprintf(stderr, " Valid modes for %s:\n", argv[currentarg]);
+					fprintf(stderr, "    0 - 24 hour sliding windows\n");
+					fprintf(stderr, "    1 - graph start from midnight\n");
 					exit(EXIT_FAILURE);
 				}
 				cfg.hourlygmode = atoi(argv[currentarg + 1]);
@@ -415,9 +417,9 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
 					if (!ishelprequest(argv[currentarg + 1]))
 						fprintf(stderr, "Error: Invalid graph selection \"%s\".\n", argv[currentarg + 1]);
-					printf(" Valid graphs for %s:\n", argv[currentarg]);
-					printf("    0 - hours\n");
-					printf("    1 - 5 minutes\n");
+					fprintf(stderr, " Valid graphs for %s:\n", argv[currentarg]);
+					fprintf(stderr, "    0 - hours\n");
+					fprintf(stderr, "    1 - 5 minutes\n");
 					exit(EXIT_FAILURE);
 				}
 				cfg.summarygraph = atoi(argv[currentarg + 1]);
@@ -431,9 +433,9 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
 					if (!ishelprequest(argv[currentarg + 1]))
 						fprintf(stderr, "Error: Invalid graph selection \"%s\".\n", argv[currentarg + 1]);
-					printf(" Valid graphs for %s:\n", argv[currentarg]);
-					printf("    0 - hours\n");
-					printf("    1 - 5 minutes\n");
+					fprintf(stderr, " Valid graphs for %s:\n", argv[currentarg]);
+					fprintf(stderr, "    0 - hours\n");
+					fprintf(stderr, "    1 - 5 minutes\n");
 					exit(EXIT_FAILURE);
 				}
 				cfg.summarygraph = atoi(argv[currentarg + 1]);
@@ -441,6 +443,27 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 					printf("Summary graph changed: %d\n", cfg.summarygraph);
 				currentarg++;
 			}
+		} else if ((strcmp(argv[currentarg], "--95th") == 0) || (strcmp(argv[currentarg], "--95") == 0) || (strcmp(argv[currentarg], "--95%") == 0)) {
+			cfg.qmode = 130;
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
+				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 2 || atoi(argv[currentarg + 1]) < 0) {
+					if (!ishelprequest(argv[currentarg + 1]))
+						fprintf(stderr, "Error: Invalid parameter \"%s\".\n", argv[currentarg + 1]);
+					fprintf(stderr, " Valid parameters for %s:\n", argv[currentarg]);
+					fprintf(stderr, "    0 - rx\n");
+					fprintf(stderr, "    1 - tx\n");
+					fprintf(stderr, "    2 - total\n");
+					exit(EXIT_FAILURE);
+				}
+				cfg.qmode += atoi(argv[currentarg + 1]);
+				currentarg++;
+			} else {
+				fprintf(stderr, "Error: Mandatory mode parameters not given for %s:\n", argv[currentarg]);
+				fprintf(stderr, "    0 - rx\n");
+				fprintf(stderr, "    1 - tx\n");
+				fprintf(stderr, "    2 - total\n");
+				exit(EXIT_FAILURE);
+			}
 		} else if ((strcmp(argv[currentarg], "-nh") == 0) || (strcmp(argv[currentarg], "--noheader")) == 0) {
 			ic->showheader = 0;
 		} else if ((strcmp(argv[currentarg], "-ne") == 0) || (strcmp(argv[currentarg], "--noedge")) == 0) {
@@ -452,9 +475,9 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
 					if (!ishelprequest(argv[currentarg + 1]))
 						fprintf(stderr, "Error: Invalid parameter \"%s\".\n", argv[currentarg + 1]);
-					printf(" Valid parameters for %s:\n", argv[currentarg]);
-					printf("    0 - bytes\n");
-					printf("    1 - bits\n");
+					fprintf(stderr, " Valid parameters for %s:\n", argv[currentarg]);
+					fprintf(stderr, "    0 - bytes\n");
+					fprintf(stderr, "    1 - bits\n");
 					exit(EXIT_FAILURE);
 				}
 				cfg.rateunit = atoi(argv[currentarg + 1]);
@@ -502,12 +525,32 @@ void parseargs(IPARAMS *p, IMAGECONTENT *ic, int argc, char **argv)
 				fprintf(stderr, "Error: Invalid or missing parameter for %s.\n", argv[currentarg]);
 				exit(EXIT_FAILURE);
 			}
+		} else if ((strcmp(argv[currentarg], "-ic") == 0) || strcmp(argv[currentarg], "--invert-colors") == 0) {
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
+				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 2 || atoi(argv[currentarg + 1]) < 0) {
+					if (!ishelprequest(argv[currentarg + 1]))
+						fprintf(stderr, "Error: Invalid parameter \"%s\".\n", argv[currentarg + 1]);
+					fprintf(stderr, " Valid parameters for %s:\n", argv[currentarg]);
+					fprintf(stderr, "    0 - no color inversion\n");
+					fprintf(stderr, "    1 - invert all colors except rx and tx\n");
+					fprintf(stderr, "    2 - invert all colors\n");
+					exit(EXIT_FAILURE);
+				}
+				ic->invert = atoi(argv[currentarg + 1]);
+				if (debug)
+					printf("Invert colors changed: %d\n", ic->invert);
+				currentarg++;
+			} else {
+				ic->invert = !ic->invert;
+				if (debug)
+					printf("Invert colors changed: %d\n", ic->invert);
+			}
 		} else if ((strcmp(argv[currentarg], "-v") == 0) || (strcmp(argv[currentarg], "--version")) == 0) {
 			printf("vnStat image output %s by Teemu Toivola <tst at iki dot fi> (SQLite %s, LibGD %d.%d.%d)\n", getversion(), sqlite3_libversion(), GD_MAJOR_VERSION, GD_MINOR_VERSION, GD_RELEASE_VERSION);
 			exit(EXIT_SUCCESS);
 		} else {
 			if (argv[currentarg][0] == '-') {
-				printf("Unknown parameter \"%s\". Use --help for help.\n", argv[currentarg]);
+				fprintf(stderr, "Unknown parameter \"%s\". Use --help for help.\n", argv[currentarg]);
 				exit(EXIT_FAILURE);
 			} else {
 				if (strchr(argv[currentarg], '+') != NULL) {
@@ -691,13 +734,13 @@ void showsupportedfileextensions(void)
 	int i;
 	const char *extensions[] = {".avif", ".bmp", ".gif", ".heif", ".heix", ".jpeg", ".jpg", ".png", ".tga", ".tif", ".tiff", ".wbmp", ".webp", ".xbm", ".xpm"};
 
-	printf("Supported image format file extensions in current environment:\n");
+	fprintf(stderr, "Supported image format file extensions in current environment:\n");
 
 	for (i = 0; i < 15; i++) {
 		if (gdSupportsFileType(extensions[i], 1)) {
-			printf("%s ", extensions[i]);
+			fprintf(stderr, "%s ", extensions[i]);
 		}
 	}
-	printf("\n");
+	fprintf(stderr, "\n");
 }
 #endif
