@@ -3,6 +3,7 @@
 #include "database_tests.h"
 #include "dbaccess.h"
 #include "ifinfo.h"
+#include "percentile.h"
 #include "dbsql.h"
 #include "dbshow.h"
 #include "dbxml.h"
@@ -188,12 +189,10 @@ START_TEST(database_outputs_do_not_crash)
 		showdb("something", 5, "", "");
 		showdb("something", 6, "", "");
 		showdb("something", 7, "", "");
-		showdb("something", 8, "", "");
 		showdb("something", 9, "", "");
-		showdb("something", 10, "", "");
 		showdb("something", 11, "", "");
 		showdb("something", 12, "", "");
-		showdb("nothing", 0, "", "");
+		showdb("something", 13, "", "");
 	}
 
 	xmlheader();
@@ -205,10 +204,10 @@ START_TEST(database_outputs_do_not_crash)
 	showxml("something", 'f', "", "");
 	showxml("something", 'a', "", "");
 	showxml("something", 's', "", "");
-	showxml("nothing", 'a', "", "");
+	showxml("something", 'p', "", "");
 	xmlfooter();
 
-	jsonheader();
+	jsonheader(JSONVERSION);
 	showjson("something", 0, 'd', "", "");
 	showjson("something", 0, 'm', "", "");
 	showjson("something", 0, 't', "", "");
@@ -218,7 +217,7 @@ START_TEST(database_outputs_do_not_crash)
 	showjson("something", 0, 'a', "", "");
 	showjson("something", 1, 'a', "", "");
 	showjson("something", 0, 's', "", "");
-	showjson("nothing", 0, 'a', "", "");
+	showjson("something", 0, 'p', "", "");
 	jsonfooter();
 
 	ret = db_close();
@@ -226,6 +225,84 @@ START_TEST(database_outputs_do_not_crash)
 }
 END_TEST
 
+START_TEST(database_showdb_exists_for_invalid_interface)
+{
+	int ret, i;
+
+	ret = db_open_rw(1);
+	ck_assert_int_eq(ret, 1);
+	ret = db_addinterface("something");
+	ck_assert_int_eq(ret, 1);
+
+	for (i = 1; i < 100; i++) {
+		ret = db_addtraffic_dated("something", (uint64_t)i * 1234, (uint64_t)i * 2345, (uint64_t)i * 85000);
+		ck_assert_int_eq(ret, 1);
+	}
+
+	ret = db_setupdated("something", (time_t)i * 85000);
+	ck_assert_int_eq(ret, 1);
+
+	suppress_output();
+
+	showdb("nothing", 0, "", "");
+
+	ret = db_close();
+	ck_assert_int_eq(ret, 1);
+}
+END_TEST
+
+START_TEST(database_showjson_exists_for_invalid_interface)
+{
+	int ret, i;
+
+	ret = db_open_rw(1);
+	ck_assert_int_eq(ret, 1);
+	ret = db_addinterface("something");
+	ck_assert_int_eq(ret, 1);
+
+	for (i = 1; i < 100; i++) {
+		ret = db_addtraffic_dated("something", (uint64_t)i * 1234, (uint64_t)i * 2345, (uint64_t)i * 85000);
+		ck_assert_int_eq(ret, 1);
+	}
+
+	ret = db_setupdated("something", (time_t)i * 85000);
+	ck_assert_int_eq(ret, 1);
+
+	suppress_output();
+
+	showjson("nothing", 0, 'a', "", "");
+
+	ret = db_close();
+	ck_assert_int_eq(ret, 1);
+}
+END_TEST
+
+START_TEST(database_showxml_exists_for_invalid_interface)
+{
+	int ret, i;
+
+	ret = db_open_rw(1);
+	ck_assert_int_eq(ret, 1);
+	ret = db_addinterface("something");
+	ck_assert_int_eq(ret, 1);
+
+	for (i = 1; i < 100; i++) {
+		ret = db_addtraffic_dated("something", (uint64_t)i * 1234, (uint64_t)i * 2345, (uint64_t)i * 85000);
+		ck_assert_int_eq(ret, 1);
+	}
+
+	ret = db_setupdated("something", (time_t)i * 85000);
+	ck_assert_int_eq(ret, 1);
+
+	suppress_output();
+
+	showxml("nothing", 'a', "", "");
+
+	ret = db_close();
+	ck_assert_int_eq(ret, 1);
+}
+END_TEST
+
 START_TEST(database_outputs_do_not_crash_without_traffic)
 {
 	int ret, i;
@@ -253,12 +330,10 @@ START_TEST(database_outputs_do_not_crash_without_traffic)
 		showdb("something", 5, "", "");
 		showdb("something", 6, "", "");
 		showdb("something", 7, "", "");
-		showdb("something", 8, "", "");
 		showdb("something", 9, "", "");
-		showdb("something", 10, "", "");
 		showdb("something", 11, "", "");
 		showdb("something", 12, "", "");
-		showdb("nothing", 0, "", "");
+		showdb("something", 13, "", "");
 	}
 
 	xmlheader();
@@ -273,7 +348,7 @@ START_TEST(database_outputs_do_not_crash_without_traffic)
 
 	xmlfooter();
 
-	jsonheader();
+	jsonheader(JSONVERSION);
 	showjson("something", 0, 'd', "", "");
 	showjson("something", 0, 'm', "", "");
 	showjson("something", 0, 't', "", "");
@@ -283,7 +358,6 @@ START_TEST(database_outputs_do_not_crash_without_traffic)
 	showjson("something", 0, 'a', "", "");
 	showjson("something", 1, 'a', "", "");
 	showjson("something", 0, 's', "", "");
-	showjson("nothing", 0, 'a', "", "");
 	jsonfooter();
 
 	ret = db_close();
@@ -312,12 +386,10 @@ START_TEST(database_outputs_do_not_crash_without_data)
 		showdb("something", 5, "", "");
 		showdb("something", 6, "", "");
 		showdb("something", 7, "", "");
-		showdb("something", 8, "", "");
 		showdb("something", 9, "", "");
-		showdb("something", 10, "", "");
 		showdb("something", 11, "", "");
 		showdb("something", 12, "", "");
-		showdb("nothing", 0, "", "");
+		showdb("something", 13, "", "");
 	}
 
 	xmlheader();
@@ -332,7 +404,7 @@ START_TEST(database_outputs_do_not_crash_without_data)
 
 	xmlfooter();
 
-	jsonheader();
+	jsonheader(JSONVERSION);
 	showjson("something", 0, 'd', "", "");
 	showjson("something", 0, 'm', "", "");
 	showjson("something", 0, 't', "", "");
@@ -342,7 +414,6 @@ START_TEST(database_outputs_do_not_crash_without_data)
 	showjson("something", 0, 'a', "", "");
 	showjson("something", 1, 'a', "", "");
 	showjson("something", 0, 's', "", "");
-	showjson("nothing", 0, 'a', "", "");
 	jsonfooter();
 
 	ret = db_close();
@@ -377,12 +448,9 @@ START_TEST(database_outputs_do_not_crash_without_data_if_totals_are_wrong)
 		showdb("something", 5, "", "");
 		showdb("something", 6, "", "");
 		showdb("something", 7, "", "");
-		showdb("something", 8, "", "");
 		showdb("something", 9, "", "");
-		showdb("something", 10, "", "");
 		showdb("something", 11, "", "");
 		showdb("something", 12, "", "");
-		showdb("nothing", 0, "", "");
 	}
 
 	xmlheader();
@@ -397,7 +465,7 @@ START_TEST(database_outputs_do_not_crash_without_data_if_totals_are_wrong)
 
 	xmlfooter();
 
-	jsonheader();
+	jsonheader(JSONVERSION);
 	showjson("something", 0, 'd', "", "");
 	showjson("something", 0, 'm', "", "");
 	showjson("something", 0, 't', "", "");
@@ -407,7 +475,6 @@ START_TEST(database_outputs_do_not_crash_without_data_if_totals_are_wrong)
 	showjson("something", 0, 'a', "", "");
 	showjson("something", 1, 'a', "", "");
 	showjson("something", 0, 's', "", "");
-	showjson("nothing", 0, 'a', "", "");
 	jsonfooter();
 
 	ret = db_close();
@@ -755,6 +822,30 @@ START_TEST(showalert_shows_nothing_with_none_type)
 }
 END_TEST
 
+START_TEST(showalert_shows_nothing_in_json_with_none_type)
+{
+	int ret;
+
+	defaultcfg();
+	cfg.qmode = 10;
+
+	ret = db_open_rw(1);
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addinterface("something");
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addtraffic("something", 100, 200);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_None, AC_Total, 42);
+	ck_assert_int_eq(ret, 0);
+
+	ret = db_close();
+	ck_assert_int_eq(ret, 1);
+}
+END_TEST
+
 START_TEST(showalert_can_alert_on_limit_and_show_things)
 {
 	int ret;
@@ -771,19 +862,74 @@ START_TEST(showalert_can_alert_on_limit_and_show_things)
 	ret = db_setalias("something", "anything");
 	ck_assert_int_eq(ret, 1);
 
-	ret = db_addtraffic("something", 100, 200);
+	ret = db_addtraffic("something", 1000, 2000);
 	ck_assert_int_eq(ret, 1);
 
-	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Hour, AC_Total, 250);
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Hour, AC_Total, 2500);
 	ck_assert_int_eq(ret, 1);
 
-	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Day, AC_Total, 250);
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Day, AC_Total, 2500);
 	ck_assert_int_eq(ret, 1);
 
-	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Month, AC_Total, 250);
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Month, AC_Total, 2500);
 	ck_assert_int_eq(ret, 1);
 
-	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Year, AC_Total, 250);
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Year, AC_Total, 2500);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Percentile, AC_RX, 2);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Percentile, AC_TX, 4);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Percentile, AC_Total, 5);
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_close();
+	ck_assert_int_eq(ret, 1);
+}
+END_TEST
+
+START_TEST(showalert_can_alert_on_limit_and_show_things_in_json)
+{
+	int ret;
+
+	defaultcfg();
+	cfg.qmode = 10;
+	suppress_output();
+
+	ret = db_open_rw(1);
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addinterface("something");
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_setalias("something", "anything");
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addtraffic("something", 1000, 2000);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Hour, AC_Total, 2500);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Day, AC_Total, 2500);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Month, AC_Total, 2500);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Year, AC_Total, 2500);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Percentile, AC_RX, 2);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Percentile, AC_TX, 4);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Percentile, AC_Total, 5);
 	ck_assert_int_eq(ret, 1);
 
 	ret = db_close();
@@ -842,6 +988,58 @@ START_TEST(showalert_limit_and_conditions_matter)
 }
 END_TEST
 
+START_TEST(showalert_limit_and_conditions_matter_also_for_json_output)
+{
+	int ret;
+
+	defaultcfg();
+	cfg.qmode = 10;
+	suppress_output();
+
+	ret = db_open_rw(1);
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addinterface("something");
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addtraffic("something", 100, 200);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Year, AC_Total, 100);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Year, AC_Total, 280);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Year, AC_Total, 350);
+	ck_assert_int_eq(ret, 0);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Year, AC_Total, 300);
+	ck_assert_int_eq(ret, 0);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Year, AC_RX, 80);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Year, AC_RX, 150);
+	ck_assert_int_eq(ret, 0);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Year, AC_TX, 180);
+	ck_assert_int_eq(ret, 1);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Year, AC_TX, 250);
+	ck_assert_int_eq(ret, 0);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Year, AC_TX, 350);
+	ck_assert_int_eq(ret, 0);
+
+	ret = showalert("something", AO_Always_Output, AE_Exit_1_On_Limit, AT_Year, AC_TX, 200);
+	ck_assert_int_eq(ret, 0);
+
+	ret = db_close();
+	ck_assert_int_eq(ret, 1);
+}
+END_TEST
+
 void add_database_tests(Suite *s)
 {
 	TCase *tc_db = tcase_create("Database");
@@ -859,6 +1057,9 @@ void add_database_tests(Suite *s)
 	tcase_add_test(tc_db, validatedb_with_invalid_totals);
 	tcase_add_test(tc_db, validatedb_with_top10_use);
 	tcase_add_test(tc_db, database_outputs_do_not_crash);
+	tcase_add_exit_test(tc_db, database_showdb_exists_for_invalid_interface, 1);
+	tcase_add_exit_test(tc_db, database_showjson_exists_for_invalid_interface, 1);
+	tcase_add_exit_test(tc_db, database_showxml_exists_for_invalid_interface, 1);
 	tcase_add_test(tc_db, database_outputs_do_not_crash_without_traffic);
 	tcase_add_test(tc_db, database_outputs_do_not_crash_without_data);
 	tcase_add_test(tc_db, database_outputs_do_not_crash_without_data_if_totals_are_wrong);
@@ -876,8 +1077,11 @@ void add_database_tests(Suite *s)
 	tcase_add_test(tc_db, importlegacydb_can_detect_when_database_read_fails);
 	tcase_add_test(tc_db, importlegacydb_can_import_legacy_database);
 	tcase_add_test(tc_db, showalert_shows_nothing_with_none_type);
+	tcase_add_test(tc_db, showalert_shows_nothing_in_json_with_none_type);
 	tcase_add_test(tc_db, showalert_can_alert_on_limit_and_show_things);
+	tcase_add_test(tc_db, showalert_can_alert_on_limit_and_show_things_in_json);
 	tcase_add_test(tc_db, showalert_limit_and_conditions_matter);
+	tcase_add_test(tc_db, showalert_limit_and_conditions_matter_also_for_json_output);
 	suite_add_tcase(s, tc_db);
 }
 
